﻿param(
    [switch] $GenerateNew = $true,
    [int] $WaitToStep = 200
)


$CLog = Join-Path -Path $(Resolve-Path $PSScriptRoot) -ChildPath 'CGen.log'
$GoLog = Join-Path -Path $(Resolve-Path $PSScriptRoot) -ChildPath 'GoOut.log'

$CExecutable = Join-Path -Path $(Resolve-Path $PSScriptRoot) -ChildPath 'a.exe'
$GoExecutable = Join-Path -Path $(Resolve-Path $PSScriptRoot) -ChildPath 'go-code\go-pge\go-pge.exe'


if($GenerateNew) {
    $CGenProc = Start-Process -FilePath $CExecutable -RedirectStandardOutput $CLog -WorkingDirectory $PSScriptRoot -PassThru
    $StepReached = $false
    while(-not $StepReached) {
        $CGenCont = Get-Content $CLog
        if($CGenCont | Select-String -Pattern "\*+ StepW: $WaitToStep \*+") {
            $StepReached = $true
        }
        Start-Sleep -Seconds 1
    }

    $CGenProc  | Stop-Process -ErrorAction SilentlyContinue


    $GoGenProc = Start-Process -FilePath $GoExecutable -ArgumentList "-pcfg=prob/bench/Pagie_1.cfg -evals=3 -peel=3 -iter=$WaitToStep -init=method1 -grow=method1"  -RedirectStandardOutput $GoLog -WorkingDirectory $(Join-Path -Path $PSScriptRoot -ChildPath 'go-code\go-pge') -PassThru
    $StepReached = $false
    while(-not $StepReached) {
        $GoGenCont = Get-Content $GoLog
        if($GoGenCont | Select-String -Pattern "PS\.step\(\)\s+$($WaitToStep)") {
            $StepReached = $true
        }
        Start-Sleep -Seconds 1
    }
    $GoGenProc | Stop-Process -ErrorAction SilentlyContinue
}

# Read Result

function Get-CSteps {
    param([Object[]] $Content)
    $ValueList = @{}
    $CurrentStep = 0
    foreach($Line in $Content) {
        if($Line -match "\*+ StepW: (\d+) \*+") {
            [int]$CurrentStep = $Matches[1]
            $ValueList[$CurrentStep] = $(New-Object -TypeName System.Collections.ArrayList)
        } elseif($Line -match "push\/pop \(\d+\,\d+\) \((.+)\)") {
            $ValueList[$CurrentStep].Add($Matches[1].Trim().Replace(' ', '')) | Out-Null
        }
    }
    return $ValueList
}

function Get-GoSteps {
    param([Object[]] $Content)
    $ValueList = @{}
    $CurrentStep = 0
    foreach($Line in $Content) {
        if($Line -match "PS\.step\(\)\s+(\d+)") {
            [int]$CurrentStep = $Matches[1]
            $ValueList[$CurrentStep] = $(New-Object -TypeName System.Collections.ArrayList)
        } elseif($Line -match "pop\/push\(\d+\,\d+\)\: (.+)") {
            $ValueList[$CurrentStep].Add($Matches[1].Trim().Trim('(').Trim(')').Replace(' ', '')) | Out-Null
        }
    }
    return $ValueList
}

function Compare-Results {
    param(
        [string]$First,
        [string]$Second
    )
    $FirstChars = $First.Trim().ToUpper().ToCharArray() | Sort
    $SecondChars = $Second.Trim().ToUpper().ToCharArray() | Sort

    [System.Collections.ArrayList]$LeftChars = $SecondChars.Clone()
    $FirstCharNr = 0
    foreach($FirstChar in $FirstChars) {
        $SecondChar = $SecondChars[$FirstCharNr]
        if($FirstChar -eq $SecondChar) {
            $LeftChars.Remove($FirstChar)
        } else {
            return $false
        }
        $FirstCharNr++
    }
    return [bool]($LeftChars.Count -eq 0)
}

$CContent = Get-Content -Path $CLog
$GoContent = Get-Content -Path $GoLog

$ParseC = Get-CSteps -Content $CContent
$ParseGo = Get-GoSteps -Content $GoContent

foreach($StepNr in $($ParseGo.Keys | Sort)) {
    $GoResult = $ParseGo[$StepNr]
    [System.Collections.ArrayList]$CResult = $ParseC[$StepNr + 1]

    if($CResult -eq $null -or $CResult.Count -lt 1) {
        Write-Information "Missing CStep nr. $StepNr"
        continue
    }
    if($GoResult -eq $null -or $GoResult.Count -lt 1) {
        Write-Information "Missing GoStep nr. $StepNr"
        continue
    }

    $StepResult = $true
    foreach($GoRes in $GoResult) {            # Step Results
        $State = $false
        foreach($CRes in $CResult.Clone()) {  # Compare every Result with every Step Result
            Write-Debug "$($GoRes) with $($CRes)"
            if(Compare-Results -First $GoRes -Second $CRes) {
                $State = $true
                $CResult.Remove($CRes)
                break
            }
        }
        if(-not $State) {
            $StepResult = $false
        }
    }

    if($StepResult) {
        Write-Host "$StepNr OK"
    } else {
        Write-Warning "$StepNr failed!"
    }
}
